@@ -1,5 +1,6 @@
 /*
     Copyright (C) 2011  Lasath Fernando <kde@lasath.org>
+    Copyright (C) 2013  Lasath Fernando <davidedmundson@kde.org>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -44,10 +45,12 @@ MessagesModel::MessagesModel(const Tp::AccountPtr &account, QObject *parent) :
     kDebug();
 
     QHash<int, QByteArray> roles;
-    roles[UserRole] = "user";
     roles[TextRole] = "text";
     roles[TimeRole] = "time";
     roles[TypeRole] = "type";
+    roles[SenderIdRole] = "senderId";
+    roles[SenderAliasRole] = "senderAlias";
+    roles[SenderAvatarRole] = "senderAvatar";
     setRoleNames(roles);
 
     d->account = account;
@@ -84,8 +87,9 @@ void MessagesModel::setupChannelSignals(const Tp::TextChannelPtr &channel)
             SLOT(onPendingMessageRemoved()));
 }
 
-void MessagesModel::setTextChannel(Tp::TextChannelPtr channel)
+void MessagesModel::setTextChannel(const Tp::TextChannelPtr &channel)
 {
+    Q_ASSERT(channel != d->textChannel);
     kDebug();
     setupChannelSignals(channel);
 
@@ -140,7 +144,6 @@ void MessagesModel::onMessageReceived(const Tp::ReceivedMessage &message)
         if (d->visible) {
             acknowledgeAllMessages();
         } else {
-            enqueueSelf();
             Q_EMIT unreadCountChanged(unreadCount);
         }
     }
@@ -170,20 +173,17 @@ QVariant MessagesModel::data(const QModelIndex &index, int role) const
     QVariant result;
 
     if (index.isValid()) {
-        KTp::Message requestedData = d->messages[index.row()];
+        const KTp::Message message = d->messages[index.row()];
 
         switch (role) {
-        case UserRole:
-            result = requestedData.senderAlias();
-            break;
         case TextRole:
-            result = requestedData.finalizedMessage();
+            result = message.finalizedMessage();
             break;
         case TypeRole:
-            if (requestedData.type() == Tp::ChannelTextMessageTypeAction) {
+            if (message.type() == Tp::ChannelTextMessageTypeAction) {
                 result = MessageTypeAction;
             } else {
-                if (requestedData.direction() == KTp::Message::LocalToRemote) {
+                if (message .direction() == KTp::Message::LocalToRemote) {
                     result = MessageTypeOutgoing;
                 } else {
                     result = MessageTypeIncoming;
@@ -191,7 +191,18 @@ QVariant MessagesModel::data(const QModelIndex &index, int role) const
             }
             break;
         case TimeRole:
-            result = requestedData.time();
+            result = message.time();
+            break;
+        case SenderIdRole:
+            result = message.senderId();
+            break;
+        case SenderAliasRole:
+            result = message.senderAlias();
+            break;
+        case SenderAvatarRole:
+            if (message.sender()) {
+                result = QVariant::fromValue(message.sender()->avatarPixmap());
+            }
             break;
         };
     } else {
@@ -254,15 +265,9 @@ void MessagesModel::acknowledgeAllMessages()
     kDebug() << "Conversation Visible, Acknowledging " << queue.size() << " messages.";
 
     d->textChannel->acknowledge(queue);
-    removeSelfFromQueue();
     Q_EMIT unreadCountChanged(queue.size());
 }
 
-void MessagesModel::selfDequeued()
-{
-    Q_EMIT popoutRequested();
-}
-
 void MessagesModel::setVisibleToUser(bool visible)
 {
     kDebug() << visible;
